==> nutrient-2022-0.1.20.tgz: package/src/import.ts <==
import { GroupSchema, InputConnectionSchema, NodeSchema, OutputConnectionSchema, PatchSchema, ProjectSchema, PropertySchemaMap } from "./export.js";
import Group from "./group.js";
import { INormalizer, Normalizer } from "./id_generator.js";
import Module from "./module.js";
import Node, { KeyValueMap, NodeType } from "./node.js";
import Patch, { GroupMap } from "./patch.js";
import Project from "./project.js";

const noNormalize: INormalizer = {
    get: (id: number) => {return id}
}

export const importProject = (projectSchema: ProjectSchema, normalize: boolean = false) => {
    let project = new Project(projectSchema.name);
    let normalizer = normalize ? new Normalizer(project.idGenerator) : undefined;
    project.mainPatch = project.activePatch = importPatch(projectSchema.patch, normalizer)
    return project;
}

export const importPatch = (patchSchema: PatchSchema, normalizer: INormalizer = noNormalize) => {
    let patch = new Patch(null);

    patch.groups = patchSchema.groups ? importGroups(patchSchema.groups) : {};

    // Nodes
    for (let [_, nodeSchema] of Object.entries(patchSchema.nodes)) {
        importNode(nodeSchema, patch, normalizer);
    }

    // Connections
    for (let [_, nodeSchema] of Object.entries(patchSchema.nodes)) {
        importConnections(nodeSchema, patch, normalizer);
    }
    return patch;
}

export const importGroups = (groupSchemas: GroupSchema[]) => {
    let groupMap: GroupMap = {};
    let idCounter = 0;
    for (let groupSchema of Object.values(groupSchemas)) {
        let group = new Group(idCounter++, groupSchema.name, groupSchema.comment, groupSchema.nodeIdList) 
        groupMap[group.id] = group;
    }
    return groupMap;
}

const createKeyValueMap = (keyValueMap: PropertySchemaMap | undefined) => {
    let map: {[key: string]: any} = {};
    if (keyValueMap) {
        for (let [key, value] of Object.entries(keyValueMap)) {
            map[key] = value;
        }

        return map;
    }
    return undefined;
}

export const importNode = (nodeSchema: NodeSchema, patch: Patch, normalizer: INormalizer = noNormalize): Node | Module => {
    let node = patch.createNode(
        normalizer.get(nodeSchema.id),
        nodeSchema.componentKey,
        createKeyValueMap(nodeSchema.properties),
        createKeyValueMap(nodeSchema.metadata),
    );

    // Module
    if (nodeSchema.type == NodeType.Module && nodeSchema.patch != null) {
        (<Module>node).patch = importPatch(nodeSchema.patch, normalizer);
        (<Module>node).patch.ownerModuleNode = node;
    }

    return node as (Node | Module);
}

export const importConnections = (nodeSchema: NodeSchema, patch: Patch, normalizer: INormalizer = noNormalize) => {
    let targetNode = patch.getNode(normalizer.get(nodeSchema.id));
    if (nodeSchema.inputs != null) {
        for (let [portKey, ports] of Object.entries(nodeSchema.inputs)) {
            for (let conn of ports) {
                let fromNode = patch.getNode(normalizer.get((<InputConnectionSchema>conn).fromNodeId));
                fromNode.createConnection(normalizer.get(conn.id), (<InputConnectionSchema>conn).fromPortKey, targetNode, portKey);
            }
        }
    }
    else if (nodeSchema.outputs != null) {
        for (let [portKey, ports] of Object.entries(nodeSchema.outputs)) {
            for (let conn of ports) {
                let toNode = patch.getNode(normalizer.get((<OutputConnectionSchema>conn).toNodeId));
                targetNode.createConnection(normalizer.get(conn.id), portKey, toNode, (<OutputConnectionSchema>conn).toPortKey);
            }
        }
    }
}

==> nutrient-2022-0.1.26.tgz: package/src/import.ts <==
import { GroupSchema, InputConnectionSchema, NodeSchema, OutputConnectionSchema, PatchSchema, ProjectSchema, PropertySchemaMap } from "./export.js";
import Group from "./group.js";
import { INormalizer, Normalizer } from "./id_generator.js";
import Module from "./module.js";
import Node, { KeyValueMap, NodeType } from "./node.js";
import Patch, { GroupMap } from "./patch.js";
import Project from "./project.js";

const noNormalize: INormalizer = {
    get: (id: number) => {return id}
}

export const importProject = (projectSchema: ProjectSchema, normalize: boolean = false) => {
    let project = new Project(projectSchema.name);
    let normalizer = normalize ? new Normalizer(project.idGenerator) : undefined;
    project.mainPatch = project.activePatch = importPatch(projectSchema.patch, normalizer)
    return project;
}

export const importPatch = (patchSchema: PatchSchema, normalizer: INormalizer = noNormalize) => {
    let patch = new Patch(null);

    patch.groups = patchSchema.groups ? importGroups(patchSchema.groups, normalizer) : {};

    // Nodes
    for (let [_, nodeSchema] of Object.entries(patchSchema.nodes)) {
        importNode(nodeSchema, patch, normalizer);
    }

    // Connections
    for (let [_, nodeSchema] of Object.entries(patchSchema.nodes)) {
        importConnections(nodeSchema, patch, normalizer);
    }
    return patch;
}

export const importGroups = (groupSchemas: GroupSchema[], normalizer: INormalizer = noNormalize) => {
    let groupMap: GroupMap = {};
    let idCounter = 0;
    for (let groupSchema of Object.values(groupSchemas)) {
        let nodeIdList = groupSchema.nodeIdList.map(id => normalizer.get(id));
        let group = new Group(idCounter++, groupSchema.name, groupSchema.comment, nodeIdList);
        groupMap[group.id] = group;
    }
    return groupMap;
}

const createKeyValueMap = (keyValueMap: PropertySchemaMap | undefined) => {
    let map: {[key: string]: any} = {};
    if (keyValueMap) {
        for (let [key, value] of Object.entries(keyValueMap)) {
            map[key] = value;
        }

        return map;
    }
    return undefined;
}

export const importNode = (nodeSchema: NodeSchema, patch: Patch, normalizer: INormalizer = noNormalize): Node | Module => {
    let node = patch.createNode(
        normalizer.get(nodeSchema.id),
        nodeSchema.componentKey,
        createKeyValueMap(nodeSchema.properties),
        createKeyValueMap(nodeSchema.metadata),
    );

    // Module
    if (nodeSchema.type == NodeType.Module && nodeSchema.patch != null) {
        (<Module>node).patch = importPatch(nodeSchema.patch, normalizer);
        (<Module>node).patch.ownerModuleNode = node;
    }

    return node as (Node | Module);
}

export const importConnections = (nodeSchema: NodeSchema, patch: Patch, normalizer: INormalizer = noNormalize) => {
    let targetNode = patch.getNode(normalizer.get(nodeSchema.id));
    if (nodeSchema.inputs != null) {
        for (let [portKey, ports] of Object.entries(nodeSchema.inputs)) {
            for (let conn of ports) {
                let fromNode = patch.getNode(normalizer.get((<InputConnectionSchema>conn).fromNodeId));
                fromNode.createConnection(normalizer.get(conn.id), (<InputConnectionSchema>conn).fromPortKey, targetNode, portKey);
            }
        }
    }
    else if (nodeSchema.outputs != null) {
        for (let [portKey, ports] of Object.entries(nodeSchema.outputs)) {
            for (let conn of ports) {
                let toNode = patch.getNode(normalizer.get((<OutputConnectionSchema>conn).toNodeId));
                targetNode.createConnection(normalizer.get(conn.id), portKey, toNode, (<OutputConnectionSchema>conn).toPortKey);
            }
        }
    }
}
